import Avatar from '@mui/material/Avatar'
import Button from '@mui/material/Button'
import TextField from '@mui/material/TextField'
import Link from '@mui/material/Link'
import Grid from '@mui/material/Unstable_Grid2'
import KeyIcon from '@mui/icons-material/Key'
import Typography from '@mui/material/Typography'
import Container from '@mui/material/Container'
import { useFormik } from 'formik'
import { useRouter } from 'next/router'
import * as yup from '@/utils/validation'
import * as api from '@/api'
import { ForgetPasswordInputDto } from '@/api/user.interface'
import { EmailVerifyCodeScene } from '@/api/email.interface'
import { useCountdown, COUNTDOWN_SECONDS } from '@/utils/useCountdown'
import { toast } from 'react-toastify'

export default function ForgetPassword() {
  const router = useRouter()
  const { countdown, setCountdown } = useCountdown()

  const formik = useFormik<ForgetPasswordInputDto>({
    initialValues: {
      email: '',
      password: '',
      verifyCode: '',
      token: '',
      userId: '',
    },
    validationSchema: yup.object({
      email: yup.emailSchema,
      password: yup.passwordSchema,
      verifyCode: yup.verifyCodeSchema,
    }),
    onSubmit: async (values) => {
      const res = await api.user.forgetPassword(values)
      localStorage.setItem('accessToken', res.data)
      toast.success('密码重置成功！', {
        onClose: () => router.push('/login'),
      })
    },
  })

  async function sendVerifyCode() {
    await formik.validateField('email')
    setCountdown(COUNTDOWN_SECONDS)
    try {
      const res = await api.email.sendEmailVerifyCode({
        email: formik.values.email,
        scene: EmailVerifyCodeScene.forgetPassword,
      })
      formik.setFieldValue('token', res.data.token)
      formik.setFieldValue('userId', res.data.userId)
    } catch (err) {
      setCountdown(0)
    }
  }

  return (
    <Container
      component="main"
      maxWidth="xs"
      sx={{
        marginTop: '20vh',
        display: 'flex',
        flexDirection: 'column',
        alignItems: 'center',
      }}
    >
      <Avatar sx={{ bgcolor: 'primary.main' }}>
        <KeyIcon />
      </Avatar>
      <Typography component="h1" variant="h5" mt={1} mb={3}>
        找回密码
      </Typography>
      <Grid
        container
        component="form"
        spacing={[2, 1]}
        onSubmit={formik.handleSubmit}
      >
        <Grid xs={12}>
          <TextField
            {...formik.getFieldProps('email')}
            required
            error={formik.touched.email && !!formik.errors.email}
            helperText={(formik.touched.email && formik.errors.email) || ' '}
            fullWidth
            label="邮箱"
            autoComplete="email"
            autoFocus
            sx={{ mt: 1 }}
          />
        </Grid>
        <Grid xs={8}>
          <TextField
            {...formik.getFieldProps('verifyCode')}
            required
            error={formik.touched.verifyCode && !!formik.errors.verifyCode}
            helperText={
              (formik.touched.verifyCode && formik.errors.verifyCode) || ' '
            }
            fullWidth
            label="验证码"
            autoComplete="verifyCode"
          />
        </Grid>
        <Grid xs={4} mt={1}>
          <Button
            size="large"
            fullWidth
            variant="contained"
            disabled={countdown > 0}
            onClick={sendVerifyCode}
          >
            {countdown > 0 ? countdown : '发送验证码'}
          </Button>
        </Grid>
        <Grid xs={12}>
          <TextField
            {...formik.getFieldProps('password')}
            required
            error={formik.touched.password && !!formik.errors.password}
            helperText={
              (formik.touched.password && formik.errors.password) || ' '
            }
            fullWidth
            label="新密码"
            type="password"
            autoComplete="current-password"
          />
        </Grid>
        <Grid xs={12}>
          <Button type="submit" fullWidth variant="contained">
            重置密码
          </Button>
        </Grid>
        <Grid xs={12} container justifyContent="flex-end" sx={{ mt: 2 }}>
          <Link href="/login">登录</Link>
        </Grid>
      </Grid>
    </Container>
  )
}
